@@ -1,6 +1,6 @@
 #  SEMgraph library
-#  Copyright (C) 2019-2021 Mario Grassi; Fernando Palluzzi
-#  e-mail: <fernando.palluzzi@gmail.com>
+#  Copyright (C) 2019-2021 Mario Grassi; Fernando Palluzzi; Barbara Tarantino
+#  e-mail: <mario.grassi@unipv.it>
 #  University of Pavia, Department of Brain and Behavioral Sciences
 #  Via Bassi 21, 27100 Pavia, Italy
 
@@ -33,7 +33,7 @@
 #' (multi-level optimization), "opc" (optimal community structure), "sgc"
 #' (spinglass statistical mechanics).
 #' @param HM Hidden model type. Enables the visualization of the hidden
-#' model. If set to "none" (default), no HM is visualized.
+#' model, gHM. If set to "none" (default), no gHM igraph object is saved.
 #' For each defined hidden module:
 #' (i) if \code{HM = "LV"}, a latent variable (LV) will be defined as
 #' common unknown cause acting on cluster nodes; (ii) if \code{HM = "CV"},
@@ -44,14 +44,11 @@
 #' variable (see also \code{\link[SEMgraph]{clusterScore}}).
 #' @param size Minimum number of nodes per module. By default, a minimum
 #' number of 5 nodes is required.
-#' @param verbose A logical value. If FALSE (default), the processed graphs
+#' @param verbose A logical value. If FALSE (default), the gHM igraph
 #' will not be plotted to screen, saving execution time (they will be
 #' returned in output anyway).
 #' @param ... Currently ignored.
 #'
-#' @import igraph
-#' @importFrom stats qnorm cov2cor cor hclust as.dist cutree
-#' @importFrom graphics abline
 #' @export
 #'
 #' @author Mario Grassi \email{mario.grassi@unipv.it}
@@ -65,11 +62,10 @@
 #' Stam CJ (2015). Hierarchical clustering in minimum spanning trees.
 #' Chaos 25(2): 023107. <https://doi.org/10.1063/1.4908014>
 #'
-#' @return If HM is not "none" a list of 3 objects is returned:
+#' @return If HM is not "none" a list of 2 objects is returned:
 #' \enumerate{
 #' \item "gHM", subgraph containing hidden modules as an igraph object;
-#' \item "membership", cluster membership vector for each node;
-#' \item "gHC", the list of modules as igraph objects.
+#' \item "membership", cluster membership vector for each node.
 #' }
 #' If HM is "none", only the cluster membership vector is returned.
 #'
@@ -79,8 +75,10 @@
 #'
 #' # Clustering ALS graph with WTC method and LV model
 #' G <- properties(alsData$graph)[[1]]
-#' membership <- clusterGraph(graph = G, type = "wtc", HM = "LV", verbose = TRUE)
-#'
+#' clv <- clusterGraph(graph = G, type = "wtc", HM = "LV")
+#' gplot(clv$gHM, l = "fdp")
+#' table(clv$membership)
+#' 
 clusterGraph <- function(graph, type = "wtc", HM = "none", size = 5,
                          verbose = FALSE, ...)
 {
@@ -94,9 +92,8 @@ clusterGraph <- function(graph, type = "wtc", HM = "none", size = 5,
 
 	if (type == "tahc") {
 		# Tree Agglomerative Hierarchical Clustering (TAHC)
-		mst <- minimum.spanning.tree(ug, weights = NULL, algorithm = NULL)
-		G <- distances(mst, v = V(mst), to = V(mst), mode = "all",
-		               weights = NA)
+		mst <- mst(ug, weights = NULL, algorithm = NULL)
+		G <- distances(mst, v = V(mst), to = V(mst), mode = "all", weights = NA)           
 		D <- 1 - cor(x = G, method = "spearman")
 		hMST <- hclust(as.dist(D), method = "average")
 		tahc <- cutree(hMST, h = 0.2)
@@ -130,11 +127,10 @@ clusterGraph <- function(graph, type = "wtc", HM = "none", size = 5,
 
 	K <-  length(cnames)
 	if (K == 0) return(message("WARNING: no communities with size >=", size, "."))
+	
 	if (HM == "UV") {
-		gHC <- cplot(graph, membership = membership, map = FALSE,
-		             verbose = FALSE)[-1]
+		gHC <- cplot(graph, membership = membership)[-1]
 		ftm <- Vxx <- NULL
-
 	for (i in 1:K) {
 		d <- igraph::degree(gHC[[i]], mode = "in")
 		Vx <- V(gHC[[i]])$name[d == 0]
@@ -155,7 +151,6 @@ clusterGraph <- function(graph, type = "wtc", HM = "none", size = 5,
 		V(gLM)$LV <- 0
 		V(gLM)$LV[1:K] <- 1
 		V(gLM)$color <- ifelse(V(gLM)$LV == 1, "lightblue", "yellow")
-		gHC <- NULL
 
 	} else if (HM == "CV") {
 		ftm <- data.frame(from = names(membership),
@@ -164,24 +159,23 @@ clusterGraph <- function(graph, type = "wtc", HM = "none", size = 5,
 		V(gLM)$LV <- 0
 		V(gLM)$LV[(vcount(gLM) - K + 1):vcount(gLM)] <- 1
 		V(gLM)$color <- ifelse(V(gLM)$LV == 1, "lightblue", "green")
-		gHC <- NULL
 
 	} else if (HM == "none") {
 		return( membership )
 	}
 
     if (verbose == TRUE) {
-		plot(gLM)
+		plot(gLM, l = "fdp")
 	}
 
-	return(list(gHM = gLM, membership = membership, gHC = gHC))
+	return(list(gHM = gLM, membership = membership))
 }
 
 #' @title Module scoring
 #'
 #' @description Generate factor scores, principal component scores, or
 #' projection scores of latent, composite, and unmeasured variable modules,
-#' respectively, and fit them in a SEM with exogenous group effect.
+#' respectively, and fit them with an exogenous group effect.
 #' @param graph An igraph object.
 #' @param data A matrix or data.frame. Rows correspond to subjects, and
 #' columns to graph nodes.
@@ -210,14 +204,11 @@ clusterGraph <- function(graph, type = "wtc", HM = "none", size = 5,
 #' (spinglass statistical mechanics).
 #' By default, the "wtc" method is used.
 #' @param verbose A logical value. If TRUE, intermediate graphs will be
-#' displayed during the execution. In addition, a condensed graph with
+#' displayed during the execution. In addition, a reduced graph with
 #' clusters as nodes will be fitted and showed to screen (see also
 #' \code{\link[SEMgraph]{mergeNodes}}). By default, \code{verbode = FALSE}.
 #' @param ... Currently ignored.
 #'
-#' @import igraph
-#' @import lavaan
-#' @importFrom cate factor.analysis
 #' @export
 #'
 #' @seealso
@@ -237,7 +228,7 @@ clusterGraph <- function(graph, type = "wtc", HM = "none", size = 5,
 #' \item "fit", hidden module fitting as a lavaan object;
 #' \item "membership", hidden module nodes membership;
 #' \code{\link[SEMgraph]{clusterGraph}} function;
-#' \item "dataHM", hidden module data matrix with cluster scores.
+#' \item "dataHM", data matrix with cluster scores in first columns.
 #' }
 #'
 #' @examples
@@ -249,7 +240,7 @@ clusterGraph <- function(graph, type = "wtc", HM = "none", size = 5,
 #'                   group = alsData$group,
 #'                   HM = "LV",
 #'                   type = "wtc",
-#'                   verbose = TRUE)
+#'                   verbose = FALSE)
 #' summary(C$fit)
 #' head(C$dataHM)
 #' table(C$membership)
@@ -330,7 +321,7 @@ clusterScore <- function(graph, data, group, HM = "LV", type = "wtc",
 		                verbose = verbose)
 		if(length(LXY) == 0) return(list(fit = NA, M = NA, dataHM = NA))
 		membership <- LXY[[2]]
-		gLC <- LXY[[3]]
+		gLC <- cplot(graph, membership = membership)[-1]
 		LXY <- paste0("HM", names(table(membership)))
 
 		# Unmeasured Variables(UV) model
@@ -373,19 +364,13 @@ clusterScore <- function(graph, data, group, HM = "LV", type = "wtc",
 			cat("Model converged:", fsr@Fit@converged, "\nSRMR:", NA, "\n\n")
 			fsr<- NULL
 		}
-
 	} else if (length(group) == 0) {
 		fsr <- NULL
 		dataLC <- cbind(group = rep(NA, nrow(dataY)), dataLC)
 	}
+	dataHM<- cbind(dataLC, dataY)
 
-	if (verbose == TRUE) {
-		X <- cbind(dataLC, data)
-		gM <- mergeNodes(graph, membership, HM = HM)
-		sem1 <- SEMfit(gM, X, group)
-	}
-
-	return(list(fit = fsr, membership = membership, dataHM = dataLC))
+	return(list(fit = fsr, membership = membership, dataHM = dataHM))
 }
 
 #' @title Cluster extraction utility
@@ -410,12 +395,6 @@ clusterScore <- function(graph, data, group, HM = "LV", type = "wtc",
 #' cluster.
 #' @param ... Currently ignored.
 #'
-#' @import igraph
-#' @import lavaan
-#' @importFrom stats cor approx
-#' @importFrom corpcor is.positive.definite cor.shrink
-#' @importFrom ggm fitAncestralGraph
-#' @importFrom flip flip plot
 #' @export
 #'
 #' @return List of clusters as igraph objects and fitting results for
@@ -447,57 +426,50 @@ clusterScore <- function(graph, data, group, HM = "LV", type = "wtc",
 #'
 #' }
 #'
-extractClusters <- function(graph, data, group = NULL, membership = NULL,
-                            map = FALSE, verbose = FALSE, ...)
+extractClusters<- function(graph, data, group = NULL, membership = NULL, map = FALSE, verbose = FALSE, ...) 
 {
-  if (is.null(membership)) {
-    membership <- clusterGraph(graph, type = "ebc", HM = "none",
-                               size = 5,
-                               verbose = FALSE)
-  }
-  clusters <- cplot(graph, membership, l=layout.auto, map, verbose)[-1]
-  if ("HM9999" %in% names(clusters)) N <- length(clusters) - 1
-  N <- length(clusters)
-  res <- NULL
-  lav <- list()
-  for (i in 1:N) {
-    fit <- quiet(SEMfit(clusters[[i]], data, group))
-    if (is.null(fit)) next
-    if (!is.null(group) & vcount(clusters[[i]]) > 100) {
-      dev_df <- fit$fit$ricf$dev/fit$fit$ricf$df
-      srmr <- fit$fit$fitIdx[3]
-      pv1 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-                        theta = fit$gest$Stat,
-                        tail = "positive")
-      pv2 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-                        theta = fit$gest$Stat,
-                        tail = "negative")
-    } else {
-      dev_df <- fitMeasures(fit$fit, "chisq")/fitMeasures(fit$fit, "df")
-      srmr <- fitMeasures(fit$fit, "srmr")
-      pv1 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-                        theta = fit$gest$est,
-                        tail = "positive")
-      pv2 <- Brown.test(x = fit$dataXY[, -1], p = fit$gest$pvalue,
-                        theta = fit$gest$est,
-                        tail = "negative")
-    }
-    
-    dfc <- data.frame(cluster = names(clusters)[i],
-                      N.nodes = vcount(clusters[[i]]),
-                      N.edges = ecount(clusters[[i]]),
-                      dev_df = round(dev_df, 3),
-                      srmr = round(srmr, 3),
-                      pv.act = round(pv1, 6),
-                      pv.inh = round(pv2, 6))
-    
-    res <- rbind(res, dfc)
-    lav[[i]] <- fit$fit
-  }
-  rownames(res) <- NULL
-  names(lav) <- names(clusters)
-  print(res)
-  return(list(clusters = clusters, fit = lav, dfc = res))
+	if (is.null(membership)) {
+	 membership<- clusterGraph(graph, type="ebc", HM="none", size=5, verbose=FALSE)
+	}
+	clusters <- cplot(graph, membership, l=layout.auto, map, verbose)[-1]
+	clusters <- clusters[-length(clusters)]
+	N <- length(clusters)
+	res <- NULL
+	lav <- list()
+	for (i in 1:N) {
+		cat("\r","cluster=", i, "of", N)
+		flush.console()
+		fit<- quiet(SEMfit(clusters[[i]], data, group))
+		if(is.null(fit)) next
+		if (!is.null(group) & vcount(clusters[[i]]) > 100){ 
+		 dev_df <- fit$fit$ricf$dev/fit$fit$ricf$df
+		 srmr <- fit$fit$fitIdx[3]
+		 pv1<- Brown.test(x=fit$dataXY[,-1], p=fit$gest$pvalue, theta=fit$gest$Stat, tail="positive")
+		 pv2<- Brown.test(x=fit$dataXY[,-1], p=fit$gest$pvalue, theta=fit$gest$Stat, tail="negative")
+		}else{
+		 dev_df <- fitMeasures(fit$fit, "chisq")/fitMeasures(fit$fit, "df")
+		 srmr <- fitMeasures(fit$fit, "srmr")
+		 pv1<- Brown.test(x=fit$dataXY[,-1], p=fit$gest$pvalue, theta=fit$gest$est, tail="positive")
+		 pv2<- Brown.test(x=fit$dataXY[,-1], p=fit$gest$pvalue, theta=fit$gest$est, tail="negative")
+		}
+		
+		dfc<- data.frame(
+		 cluster = names(clusters)[i],
+		 n.nodes = vcount(clusters[[i]]),
+		 n.edges = ecount(clusters[[i]]),
+		 dev_df = round(dev_df, 3),
+		 srmr = round(srmr, 3),
+		 V.pv.act = round(pv1, 6),
+		 V.pv.inh = round(pv2, 6))
+		 
+		res <- rbind(res, dfc)
+		lav <- c(lav, list(fit$fit))
+	}
+	message("\n\nFound ", nrow(res), " clusters with > 5 nodes")
+	rownames(res) <- NULL
+	names(lav) <- res$cluster
+	
+	return(list(clusters = clusters, fit = lav, dfc = res))
 }
 
 #' @title Subgraph mapping
@@ -516,9 +488,6 @@ extractClusters <- function(graph, data, group = NULL, membership = NULL,
 #' be returned in output anyway).
 #' @param ... Currently ignored.
 #'
-#' @import igraph
-#' @importFrom stats qnorm cov2cor cor hclust as.dist cutree
-#' @importFrom graphics abline
 #' @export
 #'
 #' @return The list of clusters and cluster mapping as igraph objects.
@@ -537,110 +506,170 @@ extractClusters <- function(graph, data, group = NULL, membership = NULL,
 #' membership <- clusterGraph(graph = G, type = "wtc")
 #' cplot(G, membership, map = TRUE, verbose = FALSE)
 #' cplot(G, membership, map = FALSE, verbose = TRUE)
+#' # The list of cluster graphs !
+#' cg <- cplot(G, membership); cg
 #'
 #' }
 #'
-cplot <- function(graph, membership, l = layout.auto, map = FALSE,
-                  verbose = FALSE, ...)
+cplot<- function (graph, membership, l = layout.auto, map = FALSE, verbose = FALSE, ...) 
 {
-  # Overall cluster visualization
-  V(graph)$M <- 9999
-  V(graph)$M[which(V(graph)$name %in% names(membership))] <- membership
-  if (map) {
-    V(graph)$color <- V(graph)$M + 1
-    gplot(graph)
-    Sys.sleep(3)
-  }
-  
-  # Within cluster visualization
-  M <- names(table(V(graph)$M))
-  K <- length(table(V(graph)$M))
-  vcol <- as.numeric(M) + 1
-  
-  HM <- lapply(1:K, function(x) induced_subgraph(graph,
-                                                 V(graph)$name[V(graph)$M == M[x]]))
-  
-  names(HM) <- paste0("HM", M)
-  d <- igraph::degree(graph, mode = "all")*2 + 1
-  
-  if (verbose) {
-    glv <- lapply(1:K, function(x) {
-      E(HM[[x]])$weight <- 1
-      plot(HM[[x]],
-           vertex.color = vcol[x],
-           vertex.size = d[V(HM[[x]])$name],
-           layout = l,
-           main = paste0("Hidden Module ", M[x]))
-      Sys.sleep(3)})
-  }
-  
-  return(invisible(c(list(graph = graph), HM)))
+	V(graph)$M <- 9999
+	V(graph)$M[which(V(graph)$name %in% names(membership))] <- membership
+	V(graph)$color <- V(graph)$M + 1
+	if (map) {
+		ig <- delete_edge_attr(graph, "weight")
+		plot(ig, layout = l)
+		Sys.sleep(3)
+	}
+	M <- names(table(V(graph)$M))
+	K <- length(table(V(graph)$M)) - 1
+	vcol <- as.numeric(M) + 1
+
+	HM <- lapply(1:K, function(x)
+	 induced_subgraph(graph, names(membership[membership == M[x]]))
+	)
+	HM[[K+1]] <- induced_subgraph(graph, V(graph)$name[V(graph)$M == 9999])
+	names(HM) <- paste0("HM", M)
+
+	d <- igraph::degree(graph, mode = "all") * 2 + 1
+	if (verbose) {
+		glv <- lapply(1:K, function(x) {
+			HM[[x]] <- delete_edge_attr(HM[[x]], "weight")
+			plot(HM[[x]], vertex.color = vcol[x], vertex.size = d[V(HM[[x]])$name], 
+				layout = l, main = paste0("Hidden Module ", M[x]))
+			Sys.sleep(3)
+		})
+	}
+	return(invisible(c(list(graph = graph), HM)))
 }
 
-#' @title Graph nodes merging by a user-defined membership attribute
+#' @title Graph nodes merging by a membership attribute
 #'
-#' @description Merge groups of graph nodes using a custom membership
-#' attribute (e.g., cluster membership).
-#' @param graph Network as an igraph object.
+#' @description Merge groups of graph nodes using hierarchical clustering
+#' with prototypes derived from \code{\link[protoclust]{protoclust}} or 
+#' custom membership attribute (e.g., cluster membership derived from
+#' \code{\link[SEMgraph]{clusterGraph}}).
+#' @param graph network as an igraph object.
+#' @param data A matrix or data.frame. Rows correspond to subjects, and
+#' columns to graph nodes.
+#' @param h Cutting the minimax clustering at height, h = 1 - abs(cor(j,k)),
+#' yielding a merged node (and a reduced data set) in which every node in the
+#' cluster has correlation of at least cor(j,k) with the prototype node.
+#' By default, \code{h = 0.5}, i.e. cor(j,k) = 0.5.
 #' @param membership Cluster membership. A vector of cluster membership
-#' identifiers, where vector names correspond to graph node names.
-#' Topological graph clustering can be done using
-#' \code{\link[SEMgraph]{clusterGraph}}.
-#' @param HM Hidden model label. If HM = "LV", a latent variable (LV)
-#' will be defined as common unknown cause acting on cluster nodes.
-#' If HM = "CV", cluster nodes will be considered as regressors of a
+#' identifiers as numeric values, where vector names correspond to graph
+#' node names. By default, \code{membership = NULL}.
+#' @param HM Hidden cluster label. If membership is derived from clusterGraph:
+#' HM = "LV", a latent variable (LV) will be defined as common unknown cause
+#' acting on cluster nodes. If HM = "CV", cluster nodes will be considered as regressors of a
 #' latent composite variable (CV). Finally, if HM = "UV", an unmeasured
 #' variable (UV) is defined, where source nodes of the module (i.e.,
 #' in-degree = 0) act as common regressors influencing the other nodes
-#' via an unmeasured variable.
+#' via an unmeasured variable. By default, \code{HM = NULL}
 #' @param ... Currently ignored.
 #'
-#' @import igraph
-#' @importFrom graph combineNodes
+#' @details Hierarchical clustering with prototypes (or Minmax linkage) is
+#' unique in naturally associating a node (the prototypes) with every
+#' interior node of the dendogram. Thus, for each merge we have a single
+#' representative data point for the resulting cluster (Bien, Tibshirani, 2011).
+#' These prototypes can be used to greatly enhance the interpretability of
+#' merging nodes and data reduction for SEM fitting.
+#
 #' @export
 #'
-#' @return A network with merged nodes as an igraph object.
+#' @return A list of 2 objects is returned:
+#' \enumerate{
+#' \item "gLM", A graph with merged nodes as an igraph object;
+#' \item "membership", cluster membership vector for each node.
+#' }
+#'
 #' @seealso \code{\link[SEMgraph]{clusterGraph}}
 #'
 #' @author Mario Grassi \email{mario.grassi@unipv.it}
 #'
+#' @references
+#'
+#' Bien J, Tibshirani R (2011). Hierarchical Clustering With Prototypes via
+#' Minimax Linkage. Journal of the American Statistical Association
+#' 106(495): 1075-1084. <doi:10.1198/jasa.2011.tm10183>
+#'
 #' @examples
 #'
-#' # Clustering ALS graph with WTC method
+#' # Gene memberships with prototypes with h=0.5
 #' G <- properties(alsData$graph)[[1]]
-#' membership <- clusterGraph(graph = G, type = "wtc")
-#' M <- mergeNodes(G, membership, HM = "LV")
+#' M <- mergeNodes(G, alsData$exprs, h = 0.5)
 #'
-mergeNodes <- function(graph, membership, HM, ...)
+#' # Gene memberships with EBC method and size=10
+#' m <- clusterGraph(G, type = "ebc", size = 10)
+#' M <- mergeNodes(G, alsData$exprs, membership = m, HM = "LV")
+#'
+#' # Gene memberships defined by user
+#' c1 <- c("5894", "5576", "5567", "572", "598")
+#' c2 <- c("6788", "84152", "2915", "836", "5530")
+#' c3 <- c("5603", "6300", "1432", "5600")
+#' m <- c(rep(1,5), rep(2,5), rep(3,4))
+#' names(m) <- c(c1, c2, c3)
+#' M <- mergeNodes(G, alsData$exprs, membership = m, HM = "CV")
+#'
+mergeNodes<- function(graph, data, h = 0.5, membership = NULL, HM = NULL, ...)
 {
-  # Set membership object
-  if (is.numeric(membership)) {
-    nodes <- names(membership)
-    membership <- paste0(HM, membership)
-    names(membership) <- nodes
-  }
-  
-  LM <- NULL
-  for (i in 1:length(table(membership))) { #i=1
-    m <- names(table(membership))[i]
-    LMi <- V(graph)$name[which(V(graph)$name %in% names(membership)[membership == m])]
-    LM <- c(LM, list(LMi))
-  }
-  names(LM) <- names(table(membership))
-  
-  # Visualize graph object
-  gLM <- as_graphnel(graph)
-  for (i in 1:length(LM)) {
-    gLMi <- graph::combineNodes(LM[[i]], gLM, names(LM)[i], mean)
-    gLM <- gLMi
-  }
-  
-  ig <- graph_from_graphnel(gLM)
-  if (length(V(ig)$color) == 0) V(ig)$color <- "white"
-  V(ig)$color[substr(V(ig)$name, 2, 2) == "V"] <- "orange"
-  vcol <- V(ig)$color
-  names(vcol) <- V(ig)$name
-  gplot(ig)
-  
-  return(gLM = ig)
+	# Set graph and membership object :
+	nodes <- colnames(data)[colnames(data) %in% V(graph)$name]
+	graph <- induced_subgraph(graph, vids= which(V(graph)$name %in% nodes))
+	if(is.numeric(membership)){
+	 nodes <- names(membership)
+	 membership <- paste0(HM, membership)
+	 names(membership) <- nodes
+	}else{
+	 membership <- prototype(data=data[,nodes], h=h, size=3)
+	}
+
+	LM<- NULL
+	for ( i in 1:length(table(membership)) ) {
+	 m<- names(table(membership))[i]
+	 LMi<- V(graph)$name[which(V(graph)$name %in% names(membership)[membership==m])]
+	 LM<- c(LM, list(LMi))
+	}
+	names(LM)<- names(table(membership))
+		
+	# visualize graph object :
+	gLM<- as_graphnel(graph)
+	for ( i in 1:length(LM) ) {
+	 gLMi<- graph::combineNodes(LM[[i]], gLM, names(LM)[i], mean)
+	 gLM<- gLMi
+	}
+	
+	ig<- graph_from_graphnel(gLM)
+	if( length(V(ig)$color) == 0 ) V(ig)$color<- "white"
+	if( length(HM) == 1){
+	  V(ig)$color[substr(V(ig)$name,2,2)=="V"]<- "pink"
+	} else{
+	  V(ig)$color[substr(V(ig)$name,1,1)=="p"]<- "green"
+	}
+	vcol<- V(ig)$color
+	names(vcol)<- V(ig)$name
+	V(ig)$name <- gsub("p", "", V(ig)$name)
+	gplot(ig)
+	
+	return(list(gLM=ig, membership=membership))
+}
+
+prototype<- function(data, h, size, ...)
+{
+	# hierarchical clustering with prototypes
+	D <- as.dist(1-abs(cor(data)))
+	hc <- protoclust::protoclust(D)
+	plot(hc);abline(h=h, lty=1, col="red")
+	cutd <- protoclust::protocut(hc, k=NULL, h=h)
+	protos <- hc$labels[cutd$protos]
+	cln <- sort(cutd$cl)
+	
+	# cluster membership with nodes > size
+	nrep <- as.numeric(table(cln))
+	cl <- unlist(lapply(1:length(protos), function(x) rep(paste0("p",protos[x]), nrep[x])))
+	names(cl) <- names(cln)
+	csize <- names(table(cl))[table(cl) >= size]
+	cl <- cl[cl %in% csize]
+		
+	return(membership = cl)
 }
