@@ -1,6 +1,6 @@
 #  SEMgraph library
-#  Copyright (C) 2019-2021 Mario Grassi; Fernando Palluzzi 
-#  e-mail: <fernando.palluzzi@gmail.com>
+#  Copyright (C) 2019-2021 Mario Grassi; Fernando Palluzzi; Barbara Tarantino
+#  e-mail: <mario.grassi@unipv.it>
 #  University of Pavia, Department of Brain and Behavioral Sciences
 #  Via Bassi 21, 27100 Pavia, Italy
 
@@ -80,9 +80,6 @@
 #' \item "DEG", a list with DEGs names per pathways.
 #' }
 #'
-#' @import igraph
-#' @import lavaan
-#' @importFrom stats pchisq p.adjust na.omit
 #' @export
 #'
 #' @author Mario Grassi \email{mario.grassi@unipv.it}
@@ -178,15 +175,19 @@ SEMgsa<- function(g=list(), data, group, method = "BH", alpha = 0.05, n_rep = 10
 	return( list(gsa=gsa, DEG=DEG) )
 }
 
-#' @title SEM-based differential connectivity inference (DCI)
+#' @title SEM-based differential causal inference (DCI)
 #'
 #' @description Creates a network with perturbed edges obtained from 
-#' the output of \code{\link[SEMgraph]{SEMrun}} with two-group and CGGM solver.
-#' To increase the efficiency of computations for large graphs,
-#' users can select to break the network structure into clusters and select
-#' the topological clustering method (see \code{\link[SEMgraph]{clusterGraph}}). 
-#' The function \code{\link[SEMgraph]{SEMrun}} is applied iteratively on 
-#' each cluster to obtain the graph with the full list of perturbed edges. 
+#' the output of \code{\link[SEMgraph]{SEMrun}} with two-group and CGGM
+#' solver, comparable to the algorithm 2 in Belyaeva et al (2021), or of
+#' \code{\link[SEMgraph]{SEMace}}, comparable to the procedure in
+#' Jablonski et al (2022).
+#' To increase the efficiency of computations for large graphs, users can
+#' select to break the network structure into clusters, and select the
+#' topological clustering method (see \code{\link[SEMgraph]{clusterGraph}}).
+#' The function \code{\link[SEMgraph]{SEMrun}} is applied iteratively on
+#' each cluster (with size min > 10 and max < 500) to obtain the graph
+#' with the full list of perturbed edges. 
 #' 
 #' @param graph Input network as an igraph object.
 #' @param data A matrix or data.frame. Rows correspond to subjects, and
@@ -194,29 +195,39 @@ SEMgsa<- function(g=list(), data, group, method = "BH", alpha = 0.05, n_rep = 10
 #' @param group A binary vector. This vector must be as long as the number
 #' of subjects. Each vector element must be 1 for cases and 0 for control
 #' subjects.
-#' @param C Topological clustering methods (default C = "none"). If
-#' \code{type = "tahc"}, network modules are generated using the tree
-#' agglomerative hierarchical clustering method (Yu et al., 2015).
-#' Other non-tree clustering methods from igraph package include: "wtc"
-#' (walktrap community structure with short random walks),
+#' @param type  Average Causal Effect (ACE) with two-group, "parents" (back-door)
+#' adjustement set, and "direct" effects (\code{type = "ace"}), or a topological
+#' clustering methods (default \code{type = "none"}). If \code{type = "tahc"},
+#' network modules are generated using the tree agglomerative hierarchical
+#' clustering method. Other non-tree clustering methods from igraph package
+#' include: "wtc" (walktrap community structure with short random walks),
 #' "ebc" (edge betweeness clustering), "fgc" (fast greedy method), "lbc"
 #' (label propagation method), "lec" (leading eigenvector method), "loc"
 #' (multi-level optimization), "opc" (optimal community structure), "sgc"
 #' (spinglass statistical mechanics).
 #' @param method Multiple testing correction method. One of the values
 #' available in \code{\link[stats]{p.adjust}}. By default, method is set
-#' to "BH" (i.e., Benjamini-Hochberg correction).
-#' @param alpha Gene set test significance level (default = 0.05).
+#' to "BH" (i.e., FDR multiple test correction).
+#' @param alpha Significance level (default = 0.05) for edge set selection.
 #' @param ... Currently ignored.
 #'
 #' @return An igraph object.
 #'
-#' @import igraph
-#' @import GGMncv
 #' @export
 #'
 #' @author Mario Grassi \email{mario.grassi@unipv.it}
 #'
+#' @references
+#'
+#' Belyaeva A, Squires C, Uhler C (2021). DCI: learning causal differences
+#' between gene regulatory networks. Bioinformatics, 37(18): 3067–3069.
+#' <https://doi: 10.1093/bioinformatics/btab167>
+#'
+#' Jablonski K, Pirkl M, Ćevid D, Bühlmann P, Beerenwinkel N (2022).
+#' Identifying cancer pathway dysregulations using differential
+#' causal effects. Bioinformatics, 38(6):1550–1559.
+#' <https://doi.org/10.1093/bioinformatics/btab847>
+#'
 #' @examples
 #'
 #' \dontrun{
@@ -230,7 +241,7 @@ SEMgsa<- function(g=list(), data, group, method = "BH", alpha = 0.05, n_rep = 10
 #' G <- properties(g)[[1]]; summary(G)
 #'
 #' # Create ALS network with perturbed edges using edge betweeness clustering
-#' gU<- SEMdci(G, als.npn, alsData$group, C="ebc", method="BH", alpha=0.2)
+#' gU<- SEMdci(G, als.npn, alsData$group, type="ebc", method="BH", alpha=0.2)
 #' gcU<- properties(gU)
 #'
 #' old.par <- par(no.readonly = TRUE)
@@ -243,32 +254,43 @@ SEMgsa<- function(g=list(), data, group, method = "BH", alpha = 0.05, n_rep = 10
 #'
 #' }
 #'
-SEMdci<- function(graph, data, group, C = "none", method = "BH", alpha = 0.05, ...)
+SEMdci<- function (graph, data, group, type = "none", method = "BH", alpha = 0.05, ...) 
 {
-	if (C != "none"){
-	 C <- clusterGraph(graph, type = C, size = 10)#table(C)
+	if (type == "ace") {
+	 dest <- SEMace(graph, data, group,
+					type = "parents", effect = "direct",
+					method = method, alpha = alpha,
+					boot = NULL)
+	 ftm <- data.frame(from = dest$source, to = dest$sink)
+	 return(gD = graph_from_data_frame(ftm))
+	}
+	if (type != "none") {
+	 C <- clusterGraph(graph, type = type, size = 10)
 	 K <- as.numeric(names(table(C)))
 	 gL <- NULL
-	 for (k in K){ #k=1
-	  cat( "fit cluster =", k, "\n" )
-	  g <- induced_subgraph(graph, vids= names(C)[C == k])
-	  dest <- quiet(SEMggm2(g, data, group, method, alpha)$dest)
-	  dsub <- subset(dest, dest$pvalue < alpha)
-	  ftm <- data.frame(from=dsub$rhs, to=dsub$lhs)
-	  gC <- graph_from_data_frame(ftm) #gplot(gU)
-	  if (ecount(gC) > 0) gL <- c(gL,list(gC))
+	 for (k in K) {
+		cat("fit cluster =", k, "\n")
+		g <- induced_subgraph(graph, vids = names(C)[C == k])
+		V <- sum(colnames(data) %in% V(g)$name) 
+		if (V < 10 | V > 500) next
+		dest <- quiet(SEMggm2(g, data, group)$dest)
+		dsub <- subset(dest, p.adjust(dest$pvalue, method = method) < alpha)
+		if (is.null(dsub)) next
+		ftm <- data.frame(from = dsub$rhs, to = dsub$lhs)
+		gC <- graph_from_data_frame(ftm)
+		if (ecount(gC) > 0) gL <- c(gL, list(gC))
 	 }
-	cat("Done.\n")
-	if (is.null(gL)) return(gU = make_empty_graph(n = vcount(graph)))
-	gU <- graph.union(gL)
-	}else{
-	 g <- graph
-	 dest <- quiet(SEMggm2(g, data, group, method, alpha)$dest)
-	 dsub <- subset(dest, dest$pvalue < alpha)
-	 ftm <- data.frame(from=dsub$rhs, to=dsub$lhs)
-	 gU <- graph_from_data_frame(ftm) #gplot(gU)
+	 cat("Done.\n")
+	 if (is.null(gL)) return(gD = make_empty_graph(n = length(K)))
+	 gD <- graph.union(gL)
 	}
-	return(gU)
+	else if (type == "none") {
+	 dest <- quiet(SEMrun(graph, data, group, algo = "cggm", fit = 2)$dest)
+	 dsub <- subset(dest, p.adjust(dest$pvalue, method = method) < alpha)
+	 ftm <- data.frame(from = dsub$rhs, to = dsub$lhs)
+	 gD <- graph_from_data_frame(ftm)
+	}
+	return(gD)
 }
 
 #' @title Graph properties summary and graph decomposition
@@ -281,7 +303,6 @@ SEMdci<- function(graph, data, group, C = "none", method = "BH", alpha = 0.05, .
 #' Nodes will be mapped onto variable names. 
 #' @param ... Currently ignored.
 #'
-#' @import igraph
 #' @export
 #'
 #' @return List of graph components, ordered by decreasing size (the first
@@ -306,7 +327,7 @@ properties<- function (graph, data = NULL, ...)
      ig <- induced_subgraph(graph, vids = which(V(graph)$name %in% nodes))
     }
     
-	ig <- igraph::simplify(ig, remove.loops = TRUE)
+	ig <- simplify(ig, remove.loops = TRUE)
     gcs <- igraph::decompose.graph(ig, min.vertices = 2)
     vsize <- sapply(1:length(gcs), function(x) vcount(gcs[[x]]))
 	names(vsize) <- 1:length(vsize)
@@ -353,9 +374,6 @@ properties<- function (graph, data = NULL, ...)
 #' @param psize Automatic node size (default = 80).
 #' @param ... Currently ignored.
 #'
-#' @import igraph
-#' @importFrom graph nodes edgeNames isDirected nodeRenderInfo edgeRenderInfo graphRenderInfo
-#' @importFrom Rgraphviz layoutGraph renderGraph
 #' @export
 #'
 #' @return gplot returns invisibly the graph object produced by Rgraphviz
@@ -375,8 +393,13 @@ gplot <- function(graph, l = "dot", main = "", cex.main = 1, font.main = 1,
                   w = "auto", h = "auto", psize = 80, ...)
 {
 	# Set graphNEL object
-	g <- as_graphnel(graph)
-
+	if(is_igraph(graph)){
+	 g <- as_graphnel(graph)
+	} else {
+	 g <- graph
+	 graph <- graph_from_graphnel(graph)
+	}
+	
 	vcol <- V(graph)$color
 	vshape <- V(graph)$shape
 	vsize <- V(graph)$size
@@ -434,13 +457,14 @@ gplot <- function(graph, l = "dot", main = "", cex.main = 1, font.main = 1,
 
 	if (w == "auto") w <- vsize
 	if (h == "auto") h <- vsize
-
-	graph::nodeRenderInfo(g)<- list(col = color, fill = vcol, lty = lty,
-	                                label = vlab, lwd = lwd,
+	
+	g<- Rgraphviz::layoutGraph(g, layoutType=l, edgeAttrs=list(label=elab))
+	graph::nodeRenderInfo(g)<- list(col = color, fill = vcol, label = vlab,
+	                                lwd = lwd, lty = lty,
 	                                textCol = color.txt,
 	                                fontsize = fontsize, cex = cex,
 	                                shape = vshape, width = w, height = h)
-	graph::edgeRenderInfo(g) <- list(col = ecol, lty = 1, lwd = elwd)
+	graph::edgeRenderInfo(g) <- list(col = ecol, lty = lty, lwd = elwd)
 	graph::graphRenderInfo(g) <- list(main = main, cex.main = cex.main,
 	                                  font.main = font.main)
 	Rgraphviz::renderGraph(g)
@@ -448,117 +472,6 @@ gplot <- function(graph, l = "dot", main = "", cex.main = 1, font.main = 1,
 	return(invisible(g))
 }
 
-#' @title Correlation matrix to graph
-#'
-#' @description Convert a correlation matrix to an igraph object.
-#' @param R Correlation matrix.
-#' @param n Sample size (i.e., the number of subjects).
-#' @param alpha Significance level used to compute the correlation threshold.
-#' By default, \code{alpha = 0.05}.
-#' @param method Multiple testing correction method. One of the values
-#' available in \code{\link[stats]{p.adjust}}. By default,
-#' \code{method = "none"} (i.e., no multiple test correction).
-#' See \code{\link[stats]{p.adjust}} for other correction methods.
-#' @param type Graph building method. If \code{type} is either
-#' \code{"marg"} or \code{"cond"}, marginal or conditional correlation
-#' tests will be used, respectively.
-#' If \code{type = "mst"}, input correlations are converted to distances
-#' and a minimum spanning tree is generated from the distance matrix,
-#' using Prim's algorithm (Prim, 1957).
-#' If \code{type = "tmfg"}, a triangulate maximally graph is generated
-#' from the given correlation matrix (Massara et al., 2016).
-#' @param ... Currently ignored.
-#'
-#' @import lavaan
-#' @import igraph
-#' @export
-#'
-#' @references
-#'
-#' Palluzzi F, Grassi M (2021). SEMgraph: An R Package for Causal Network
-#' Analysis of High-Throughput Data with Structural Equation Models.
-#' <arXiv:2103.08332>
-#'
-#' Massara GP, Di Matteo T and Aste T (2009). Network Filtering for Big
-#' Data: Triangulated Maximally Filtered Graph.
-#' Journal of complex Networks, 5(2): 161--178.
-#' <https://doi.org/10.1093/comnet/cnw015>
-#'
-#' Prim RC (1957). Shortest connection networks and some generalizations.
-#' Bell System Technical Journal, 36(6):1389--1401.
-#' <https://doi.org/10.1002/j.1538-7305.1957.tb01515.x>
-#'
-#' @return An igraph object.
-#'
-#' @author Mario Grassi \email{mario.grassi@unipv.it}
-#'
-#' @examples
-#'
-#' # Graphs creation
-#' C1 <- corr2graph(R = cor(log(sachs$pkc)), n = nrow(sachs$pkc),
-#'                  type = "marg",
-#'                  method = "BH")
-#' C2 <- corr2graph(R = cor(log(sachs$pkc)), n = nrow(sachs$pkc),
-#'                  type = "cond",
-#'                  method = "BH")
-#' C3 <- corr2graph(R = cor(log(sachs$pkc)), n = nrow(sachs$pkc),
-#'                  type = "mst",
-#'                  method = "BH")
-#' C4 <- corr2graph(R = cor(log(sachs$pkc)), n = nrow(sachs$pkc),
-#'                  type = "tmfg",
-#'                  method = "BH")
-#'
-#' # Graphs plots
-#' old.par <- par(no.readonly = TRUE)
-#' par(mfrow=c(2,2), mar= rep(2, 4))
-#' plot(C1, layout=layout.circle, main= "marg"); box(col="gray")
-#' plot(C2, layout=layout.circle, main= "cond"); box(col="gray")
-#' plot(C3, layout=layout.circle, main= "mst"); box(col="gray")
-#' plot(C4, layout=layout.circle, main= "tmfg"); box(col="gray")
-#' par(old.par)
-#' 
-corr2graph <- function(R, n, type = "marg", method = "none",
-                       alpha = 0.05, ...)
-{
-	# Set correlation matrix
-	p <- nrow(R)
-	q <- 0
-	if (type == "cond") {
-		q <- p - 2
-		if (corpcor::is.positive.definite(R)) {
-			K <- corpcor::cor2pcor(R)
-			rownames(K) <- colnames(K) <- rownames(R)
-		} else {
-			K <- corpcor::pcor.shrink(R, verbose = TRUE)[1:p, 1:p]
-		}
-	} else {
-		K <- R
-	}
-
-	if (type == "marg" | type == "cond") {
-		# select the correlation threshold
-		z <- abs(atanh(K[lower.tri(K)]))/sqrt(1/(n - 3 - q))
-		p.adj <- p.adjust(2*(1 - pnorm(z)), method = method)
-		Z <- min(z[which(p.adj < alpha)])
-		#Z <- qnorm(alpha/2, lower.tail = FALSE)
-		thr <- (exp(2*Z/sqrt(n - 3 - q)) - 1)/(exp(2*Z/sqrt(n - 3 - q)) + 1)
-		A0 <- ifelse(abs(K) > thr, 1, 0)
-		diag(A0) <- 0
-		del <- which(colSums(A0) == 0)
-		if (length(del) > 0) A <- A0[-del, -del] else A <- A0
-		ug <- graph_from_adjacency_matrix(A, mode = "undirected")
-	}
-    if (type == "mst") {
-		D <- diag(p) - K^2
-		gA <- graph_from_adjacency_matrix(D, mode = "undirected",
-		                                  weighted = TRUE)
-		ug <- igraph::mst(gA, algorithm = "prim")
-	}
-	if (type == "tmfg") ug <- TMFG(K)$graph
-
-	return(graph = ug)
-}
-
 #' @title lavaan model to graph
 #'
 #' @description Convert a model, specified using lavaan syntax,
@@ -570,16 +483,11 @@ corr2graph <- function(R, n, type = "marg", method = "none",
 #' @param psi Logical value. If TRUE (default) covariances will be converted
 #' into bidirected graph edges. If FALSE, covariances will be excluded from
 #' the output graph.
-#' @param format Output graph format. It can be either "igraph" (default)
-#' or "dagitty".
 #' @param verbose Logical value. If TRUE, a plot of the output graph will
 #' be generated. For large graphs, this could significantly increase
 #' computation time. If FALSE (default), graph plotting will be disabled.
 #' @param ... Currently ignored.
 #'
-#' @import lavaan
-#' @import igraph
-#' @importFrom dagitty graphLayout canonicalize
 #' @export
 #'
 #' @return An igraph object.
@@ -616,25 +524,24 @@ corr2graph <- function(R, n, type = "marg", method = "none",
 #' G1 <- lavaan2graph(model, psi = FALSE)
 #' plot(G1, layout = layout.circle)
 #'
-lavaan2graph <- function(model, directed = TRUE, psi = TRUE,
-                         format = "igraph", verbose = FALSE, ...)
+lavaan2graph<- function (model, directed = TRUE, psi = TRUE, verbose = FALSE, ...)			
 {
-	lav <- lavParTable(model, fixed.x = FALSE)
-	lavb <- subset(lav, lav$op == "~")
-	lavc <- subset(lav, lav$op == "~~" & (lav$rhs != lav$lhs))
-	ftm <- data.frame(cbind(from = lavb$rhs, to = lavb$lhs, label = lavb$label),
-	                  color="blue")
-	if (nrow(lavc) != 0 & psi == TRUE) {
-		ftmc1 <- data.frame(cbind(from = lavc$rhs, to = lavc$lhs, label = "",
-		                    color = "gray60"))
-		ftmc2 <- data.frame(cbind(from = lavc$lhs, to = lavc$rhs, label = "",
-		                    color = "gray60"))
-		ftm <- rbind(ftm, ftmc1, ftmc2)
-	}
-	graph <- graph_from_data_frame(ftm, directed = directed)
-	if (format == "dagitty") graph <- graph2dagitty(graph, verbose)
-	if (format == "igraph" & verbose) plot(graph)
-	return(graph)
+    lav <- lavParTable(model, fixed.x = FALSE)
+    lavb <- subset(lav, lav$op == "~")
+    lavc <- subset(lav, lav$op == "~~" & (lav$rhs != lav$lhs))
+	lavc<- lavc[lavc$user == 1,]
+    ftm <- data.frame(cbind(from = lavb$rhs, to = lavb$lhs, label = lavb$label), 
+        color = "black")
+    if (nrow(lavc) != 0 & psi == TRUE) {
+		ftmc1 <- data.frame(cbind(from = lavc$rhs, to = lavc$lhs, 
+            label = "", color = "gray60"))
+        ftmc2 <- data.frame(cbind(from = lavc$lhs, to = lavc$rhs, 
+            label = "", color = "gray60"))
+        ftm <- rbind(ftm, ftmc1, ftmc2)
+    }
+    graph <- graph_from_data_frame(ftm, directed = directed)
+	if (verbose) gplot(graph)
+    return(graph)
 }
 
 #' @title Graph to lavaan model
@@ -645,9 +552,6 @@ lavaan2graph <- function(model, directed = TRUE, psi = TRUE,
 #' all the input graph nodes will be included in the output model.
 #' @param ... Currently ignored.
 #'
-#' @import lavaan
-#' @import igraph
-#' @importFrom dagitty graphLayout canonicalize
 #' @export
 #'
 #' @author Mario Grassi \email{mario.grassi@unipv.it}
@@ -695,16 +599,15 @@ graph2lavaan <- function(graph, nodes = V(graph)$name, ...)
 #' @title Graph conversion from igraph to dagitty
 #'
 #' @description Convert an igraph object to a dagitty object.
-#' @param graph A graph as an igraph or dagitty object.
-#' @param canonical A logical value. If TRUE, DAG conversion is enforced
-#' (for \code{graph2dagitty} only). This argument is FALSE by default.
+#' @param graph A graph as an igraph or as an adjacency matrix.
+#' @param graphType character, is one of "dag" (default)' or "pdag".
+#' DAG can contain the directed (->) and bi-directed (<->) edges,
+#' while PDAG can contain the edges: ->, <->, and the undirected edges
+#' (--) that represent edges whose direction is not known.
 #' @param verbose A logical value. If TRUE, the output graph is shown
 #' (for \code{graph2dagitty} only). This argument is FALSE by default.
 #' @param ... Currently ignored.
 #'
-#' @import lavaan
-#' @import igraph
-#' @importFrom dagitty graphLayout canonicalize
 #' @export
 #'
 #' @author Mario Grassi \email{mario.grassi@unipv.it}
@@ -717,28 +620,63 @@ graph2lavaan <- function(graph, nodes = V(graph)$name, ...)
 #'
 #' @return A dagitty object.
 #'
-graph2dagitty <- function (graph, canonical = FALSE, verbose = FALSE, ...)
+graph2dagitty<- function (graph, graphType = "dag", verbose = FALSE, ...) 
 {
-    dg <- graph - E(graph)[which_mutual(graph)]
-	ug <- as.undirected(graph - E(graph)[!which_mutual(graph)])
-	ed <- attr(E(dg), "vnames")
+    if (!is.igraph(graph)) graph <- graph_from_adjacency_matrix(graph)
+	dg <- graph - E(graph)[which_mutual(graph)]
+    ug <- as.undirected(graph - E(graph)[!which_mutual(graph)])
+    ed <- attr(E(dg), "vnames")
     eb <- attr(E(ug), "vnames")
     de <- paste(gsub("\\|", "->", ed), collapse = "\n")
-
     if (length(eb) == 0) {
-		dagi <- paste0("dag {\n", de, "\n}")
-    } else {
-		be <- paste(gsub("\\|", "<->", eb), collapse = "\n")
-		dagi <- paste0("dag {\n", de, "\n", be, "\n}")
+        dagi <- paste0("dag {\n", de, "\n}")
     }
-
-	if (verbose) plot(dagitty::graphLayout(dagi))
-	if (canonical) {
-		dagi <- dagitty::canonicalize(dagi)
-		if (verbose) plot(dagitty::graphLayout(dagi$g))
-		return(dagi$g)
+    else {
+		be <- paste(gsub("\\|", "<->", eb), collapse = "\n")
+        dagi <- paste0("dag {\n", de, "\n", be, "\n}")
+    }
+    if (graphType == "pdag"){
+		dagi <- gsub("<->", "--", dagi)
+		dagi <- gsub("dag", "pdag", dagi) #cat(dagy)
 	}
-    return(dagi)
+	if (verbose) plot(dagitty::graphLayout(dagi))
+  	return(dagi)
+}
+
+#' @title Graph conversion from dagitty to igraph
+#'
+#' @description Convert a dagitty object to a igraph object.
+#' @param dagi A graph as a dagitty object ("dag" or "pdag").
+#' @param verbose A logical value. If TRUE, the output graph is shown
+#' (for \code{graph2dagitty} only). This argument is FALSE by default.
+#' @param ... Currently ignored.
+#' 
+#' @export
+#'
+#' @author Mario Grassi \email{mario.grassi@unipv.it}
+#'
+#' @examples
+#'
+#' # Conversion from igraph to dagitty  (and viceversa)
+#' dagi <- graph2dagitty(sachs$graph, verbose = TRUE)
+#' graph <- dagitty2graph(dagi, verbose = TRUE)
+#'
+#' @return An igraph object.
+#'
+dagitty2graph<- function(dagi, verbose = FALSE, ...) 
+{
+	# edges to ftm
+	edges<- dagitty::edges(dagi)
+	dsel<- which(edges$e == "->")
+	d1<- data.frame(from=edges$v[dsel], to=edges$w[dsel])
+	bsel<- which(edges$e == "<->" | edges$e == "--")
+	b1<- data.frame(from=edges$v[bsel], to=edges$w[bsel])
+	b2<- data.frame(from=edges$w[bsel], to=edges$v[bsel])
+	# ftm to graph
+	ftm<- rbind(d1,b1,b2)
+	graph<- graph_from_data_frame(ftm)
+	if (verbose) gplot(graph)
+	return(graph)
 }
 
 #' @title Convert directed graphs to directed acyclic graphs (DAGs)
@@ -762,7 +700,6 @@ graph2dagitty <- function (graph, canonical = FALSE, verbose = FALSE, ...)
 #' a BAP is generated merging the output DAG and the bidirected edges
 #' from the input graph.
 #'
-#' @import igraph
 #' @export
 #'
 #' @return A DAG as an igraph object.
@@ -778,84 +715,83 @@ graph2dagitty <- function (graph, canonical = FALSE, verbose = FALSE, ...)
 #' gplot(dag, main = "Output DAG")
 #' par(old.par)
 #'
-graph2dag <- function(graph, data, bap = FALSE, time.limit = Inf, ...)
+graph2dag<- function(graph, data, bap = FALSE, time.limit = Inf, ...)
 {
-	if (is_dag(graph)) return(dag = graph)
-
-	# Graph weighting by edge pvalues (r2z)
-	graph <- weightGraph(graph, data, group = NULL, seed = "none")
-	E(graph)$weight <- 1/(-log(E(graph)$pv))
+	if (is_dag(graph)) return(dag=graph)
+	# graph weighting by edge pvalues (r2z)
+	graph <- weightGraph(graph, data)
+	E(graph)$weight <- 1/(-log(E(graph )$pv))
 	ftm <- as_data_frame(graph)
 	wE <- ftm$weight
-	names(wE) <- paste0(ftm[, 1], ":", ftm[, 2])
-
-	# Delete all mutual edges <-> , i.e. <- & ->
-	ig <- graph - E(graph)[which_mutual(graph)]
-	if (is_dag(ig)) {
-		cat("DAG conversion: TRUE\n")
-		if (bap == TRUE) return(bap = graph)
-		return(dag = ig)
+	names(wE) <- paste0(ftm[,1],":",ftm[,2])
+	# delete all mutual edges <-> , i.e. <- & -> 
+	ig <- graph - E(graph)[which_mutual(graph )]
+	if (is_dag(ig) & bap == FALSE) {
+	 cat("DAG conversion : TRUE\n")
+	 return(dag = ig)
 	}
-
-	# Subgraph isomorphism algorithm to detect all cycles of a given length
+	if (is_dag(ig) & bap == TRUE) {
+	 cat("BAP conversion : TRUE\n")
+	 return(bap = graph)
+	}
+	
+	# subgraph isomorphism algorithm to detect all cycles of a given length
+	# time limit: CPU time for the computation, in seconds (defaults=Inf)
 	find.cycles <- function(graph, k, time.limit) {
-		ring <- graph.ring(k, TRUE)
-		subgraph_isomorphisms(ring, graph, "lad", time.limit = time.limit)
+	 ring <- graph.ring(k, TRUE)
+	 subgraph_isomorphisms(ring, graph, "lad", time.limit=time.limit)
 	}
-
-	# Function that identifies the right subisomorphisms to keep
+	# function that identifies the right subisomorphisms to keep
 	subisomorphism_rm_permutation <- function(si) {
-		is_first_min <- function(x) {
-			return(x[1] == min(x))
-		}
-		sel <- lapply(si, is_first_min)
-		return(si[unlist(sel)])
+	 is_first_min <- function(x) { return(x[1] == min(x)) }
+	 sel <- lapply(si, is_first_min)
+	 return(si[unlist(sel)])
+	}
+	# function that search max(weight) edge on each cycle
+	max_edges <- function(x){
+	 Ec <- vector()
+	 for (i in 1:(length(x)-1)) Ec<- c(Ec, paste0(x[i],":",x[i+1]))
+	 Ew <- wE[which(names(wE) %in% Ec)]
+	 return(unlist(strsplit(names(Ew)[which(Ew == max(Ew))],":")))
 	}
 
-	# Function that search max(weight) edge on each cycle
-	max_edges <- function(x) {
-		Ec <- vector()
-		for (i in 1:(length(x)-1)) Ec <- c(Ec, paste0(x[i], ":", x[i + 1]))
-		Ew <- wE[which(names(wE) %in% Ec)]
-		if(length(Ew) == 0) Ew <- wE[1]
-		return(unlist(strsplit(names(Ew)[which(Ew == max(Ew))], ":")))
+	for (k in 3:vcount(ig)){ #k=6
+	 # find all cycles with k vertices(edges)
+	 l <- find.cycles(ig, k, time.limit)
+	 # remove permutations
+	 l <- subisomorphism_rm_permutation(si=l)
+	 # extract the vertices in each cycle
+	 if (length(l) == 0) next
+	 cycles <- lapply(1:length(l), function(x) names(l[[x]]))
+	 # edges with max(weight)
+	 l <- unique(lapply(cycles, max_edges))
+	 E1 <- unlist(lapply(l, function(x) paste0(x[1],"|",x[2])))
+	 E0 <- attr(E(ig), "vnames")
+	 ig <- delete_edges(ig, which(E0 %in% E1))
 	}
-
-	for (k in 3:vcount(graph)) {
-		# Find all cycles with k vertices(edges)
-		l <- find.cycles(ig, k, time.limit)
-		# Remove permutations
-		l <- subisomorphism_rm_permutation(si = l)
-		# Extract the vertices in each cycle
-		if (length(l) == 0) next
-		cycles <- lapply(1:length(l), function(x) names(l[[x]]))
-		# Edges with max(weight)
-		l <- unique(lapply(cycles, max_edges))
-		E1 <- unlist(lapply(l, function(x) paste0(x[1], "|", x[2])))
-		E0 <- attr(E(ig), "vnames")
-		ig <- delete_edges(ig, which(E0 %in% E1))
+	if (bap == FALSE) {
+	 cat("DAG conversion :", is_dag(ig),"\n")
+	 return(dag = ig)
 	}
-	cat("DAG conversion :", is_dag(ig), "\n")
-
-	# Add all mutual edges <-> , i.e. <- & ->
-	if (bap == TRUE) {
-		e <- as_edgelist(graph - E(graph)[!which_mutual(graph)])
-		return(bap = add_edges(ig, as.vector(t(e))))
+    if (bap == TRUE) {
+	 cat("BAP conversion :", is_dag(ig),"\n")
+	 # add all mutual edges <-> , i.e. <- & ->
+	 e <- as_edgelist(graph-E(graph)[!which_mutual(graph)])
+	 return(bap = add_edges(ig, as.vector(t(e))))
 	}
-
-	return(graph = ig)
 }
 
 #' @title Assign edge orientation of an undirected graph
 #'
 #' @description Assign edge orientation of an undirected graph
-#' through a given reference directed graph.
+#' through a given reference directed graph. The vertex (color)
+#' and edge (color, width and weight) attributes of the input
+#' undirected graph are preserved in the output directed graph. 
 #'
 #' @param ug An undirected graph as an igraph object.
 #' @param dg A directed reference graph.
 #' @param ... Currently ignored.
 #'
-#' @import igraph
 #' @export
 #'
 #' @return A directed graph as an igraph object.
@@ -863,7 +799,7 @@ graph2dag <- function(graph, data, bap = FALSE, time.limit = Inf, ...)
 #' @examples
 #'
 #' # Graphs definition
-#' G0 <- corr2graph(R = cor(log(sachs$pkc)), n = nrow(sachs$pkc), type = "marg")
+#' G0 <- as.undirected(sachs$graph)
 #'
 #' # Reference graph-based orientation
 #' G1 <- orientEdges(ug = G0, dg = sachs$graph)
@@ -878,29 +814,66 @@ graph2dag <- function(graph, data, bap = FALSE, time.limit = Inf, ...)
 orientEdges<- function(ug, dg, ...)
 {
 	if (is_directed(ug)){
-	 return(message("ERROR: the input graph is a Directed graph !"))
+	 return(message(" ERROR: the input graph is a Directed graph !"))
 	}
 	if (!is_directed(dg)){
-	 return(message("ERROR: the reference graph is an Undirected graph !"))
+	 return(message(" ERROR: the reference graph is an UNdirected graph !"))
 	}
+	
+	# graph edge comparison:
 	mg <- as.directed(ug, mode = "mutual")
 	exy0 <- attr(E(mg), "vnames")
 	exy1 <- attr(E(dg)[which_mutual(dg) == FALSE], "vnames")
 	exy2 <- exy0[which(exy0 %in% exy1)]
 	if (length(exy2) == 0) return(graph = mg)
 	str2 <- strsplit(exy2,"\\|")
-	ftm2 <- matrix(unlist(str2),nrow=length(str2),byrow=TRUE)
-	g2 <- graph_from_edgelist(ftm2, directed=TRUE)
-	ug0 <- difference(ug, as.undirected(g2))
+	ftm1 <- matrix(unlist(str2),nrow=length(str2),byrow=TRUE)
+	g1 <- graph_from_edgelist(ftm1, directed=TRUE)
+	# plot(g1, layout=layout.circle)
+	ug0 <- difference(ug, as.undirected(g1))
 	mg0 <- as.directed(ug0, mode = "mutual")
-	g <- graph.union(mg0, g2)
-	E1 <- attr(E(g), "vnames")
-	E0 <- attr(E(g2), "vnames")
-	E(g)$color<- ifelse(E1 %in% E0, "blue", "gray")
-	V(g)$color <- colorMatch(ug, g)[[1]]
+	
+	#output graph with attr matching:
+	g <- graph.union(mg0, g1)
+	gattr<- attrMatch(mg, g)
+	V(g)$color <- gattr[[1]]
+	E(g)$color <- gattr[[2]]
+	E(g)$width <- gattr[[3]]
+	E(g)$weight <- gattr[[4]]
+	# plot(g, layout=layout.circle)
+	
 	return(graph = g)
 }
 
+attrMatch<- function(g1, g2, ...)
+{
+	if (!is.null(V(g1)$color)) {
+	 idx<- match(V(g2)$name, V(g1)$name)
+	 Vcol<- V(g1)$color[idx]
+	}else{
+	 Vcol<- rep(NA, vcount(g2))
+	}
+	if (!is.null(E(g1)$color)) {
+	 idx<- match(attr(E(g2), "vnames"), attr(E(g1), "vnames"))
+	 Ecol<- E(g1)$color[idx]
+	}else{
+	 Ecol<- rep("gray", ecount(g2))
+	}
+	if (!is.null(E(g1)$width)) {
+	 idx<- match(attr(E(g2), "vnames"), attr(E(g1), "vnames"))
+	 Ewid<- E(g1)$width[idx]
+	}else{
+	 Ewid<- rep(1, ecount(g2))
+	}
+	if (!is.null(E(g1)$weight)) {
+	 idx<- match(attr(E(g2), "vnames"), attr(E(g1), "vnames"))
+	 Ewei<- E(g1)$weight[idx]
+	}else{
+	 Ewei<- rep(1, ecount(g2))
+	}
+	return(list(Vcol, Ecol, Ewid, Ewei))
+}
+
 #' @title Vertex and edge graph coloring on the base of fitting
 #'
 #' @description Add vertex and edge color attributes to an igraph object,
@@ -934,7 +907,6 @@ orientEdges<- function(ug, dg, ...)
 #' is given to edges with P-value < alpha. By default ewidth = c(1, 2).
 #' @param ... Currently ignored.
 #'
-#' @import igraph
 #' @export
 #'
 #' @return An igraph object with vertex and edge color and width attributes.
@@ -1045,29 +1017,6 @@ colorGraph <- function (est, graph, group, method = "none", alpha = 0.05,
 	return(graph)
 }
 
-colorMatch <- function(g1, g2, ...)
-{
-	if (!is.null(V(g1)$color)) {
-		idx <- match(V(g2)$name, V(g1)$name)
-		Vcol <- V(g1)$color[idx]
-	} else {
-		Vcol <- rep("white", vcount(g2))
-	}
-	if (!is.null(E(g1)$color)) {
-		idx <- match(attr(E(g2), "vnames"), attr(E(g1), "vnames"))
-		Ecol <- E(g1)$color[idx]
-	} else {
-		Ecol <- rep("gray60", ecount(g2))
-	}
-	if (!is.null(E(g1)$width)) {
-		idx <- match(attr(E(g2), "vnames"), attr(E(g1), "vnames"))
-		Ewid <- E(g1)$width[idx]
-	} else {
-		Ewid <- rep(1, ecount(g2))
-	}
-	return(list(Vcol, Ecol, Ewid))
-}
-
 #' @title Pairwise plotting of multivariate data
 #'
 #' @description Display a pairwise scatter plot of two datasets for a
@@ -1075,14 +1024,12 @@ colorMatch <- function(g1, g2, ...)
 #' the function displays a histogram with normal curve superposition.
 #'
 #' @param x A matrix or data.frame (n x p) of continuous data.
-#' @param y A matrix or data.frame (n x q) of continuous data.
+#' @param y A matrix or data.frame (n x p) of continuous data matched with x.
 #' @param size number of rows to be sampled (default \code{s = nrow(z)}).
 #' @param r number of rows of the plot layout (default \code{r = 4}).
 #' @param c number of columns of the plot layout (default \code{r = 4}).
 #' @param ... Currently ignored.
 #'
-#' @importFrom graphics par hist curve legend
-#' @importFrom stats dnorm
 #' @export
 #'
 #' @return No return value
@@ -1096,7 +1043,7 @@ colorMatch <- function(g1, g2, ...)
 #'
 pairwiseMatrix<- function (x, y = NULL, size = nrow(x), r = 4, c = 4, ...)
 {
-    if (r * c > ncol(x)) {
+ 	if (r * c > ncol(x)) {
   	 r <- r - 1
 	 c <- c - 1
 	 p <- 1:(r * c)
